/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/

// CellToEdge.cpp

//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// Dan Martin, Fri, Jan 14, 2000

#include <cassert>
#include "DataIterator.H"
#include "CellToEdge.H"
#include "CellToEdgeF_F.H"


#include "Timer.H"

//timercomment//extern Timer NST_Everything;
//timercomment//Timer TF_CellToEdge1  ("CellToEdge1",     NST_Everything, DIAG_TABLE4);
//timercomment//Timer TF_CellToEdge2  ("CellToEdge2",     NST_Everything, DIAG_TABLE4);
//timercomment//Timer TF_CellToEdge3  ("CellToEdge3",     NST_Everything, DIAG_TABLE4);

/// -----------------------------------------------------------
void CellToEdge(const LevelData<FArrayBox>& a_cellData,
		LevelData<FluxBox>& a_edgeData)
  
{
  // this is just a wrapper around single-gridBox version
  DataIterator dit = a_cellData.dataIterator();
  for (dit.reset(); dit.ok(); ++dit) 
    {
      CellToEdge(a_cellData[dit()], a_edgeData[dit()]);
    }
}
      
  
///
void CellToEdge(const FArrayBox& a_cellData,
		FluxBox& a_edgeData)
{

  // quick checks to make sure this will work
  //assert(a_edgeData.nComp() == a_cellData.nComp());

  //const Box& interiorBox = a_edgeData.box();

  //timercomment//TF_CellToEdge1.start();
  // loop over components -- assumption is that in cell-centered
  // data, direction changes faster than component
  for (int comp = 0; comp < a_edgeData.nComp(); comp++) 
    {
      // loop over directions
      for (int dir = 0; dir < SpaceDim; dir++) 
	{
	  const Box& edgeBox = a_edgeData[dir].box();
	  int cellcomp = SpaceDim*comp + dir;
	  FORT_CELLTOEDGE(CHF_CONST_FRA1(a_cellData, cellcomp),
			  CHF_FRA1(a_edgeData[dir], comp),
			  CHF_BOX(edgeBox),
			  CHF_CONST_INT(dir));
	}
    }
  //timercomment//TF_CellToEdge1.stop();
}


// ------------------------------------------------------------
void CellToEdge(const FArrayBox& a_cellData, FArrayBox& a_edgeData,
		const int a_dir)

{

  const Box& edgeBox = a_edgeData.box();

  //timercomment//TF_CellToEdge2.start();
  for (int comp = 0; comp < a_edgeData.nComp(); comp++) {
    int cellComp = SpaceDim*comp + a_dir;
    FORT_CELLTOEDGE(CHF_CONST_FRA1(a_cellData, cellComp),
		    CHF_FRA1(a_edgeData, comp),
		    CHF_BOX(edgeBox),
		    CHF_CONST_INT(a_dir));
  }
  //timercomment//TF_CellToEdge2.stop();

}


// ------------------------------------------------------------
void CellToEdge(const FArrayBox& a_cellData, const int a_cellComp,
		FArrayBox& a_edgeData, const int a_edgeComp,
		const int a_dir)

{

  const Box& edgeBox = a_edgeData.box();

  //timercomment//TF_CellToEdge3.start();
  FORT_CELLTOEDGE(CHF_CONST_FRA1(a_cellData,a_cellComp),
		  CHF_FRA1(a_edgeData,a_edgeComp),
		  CHF_BOX(edgeBox),
		  CHF_CONST_INT(a_dir));
  //timercomment//TF_CellToEdge3.stop();
}


